#include "ChaCha.hpp"
using namespace cipher;

ChaCha::~ChaCha()
{
	memset(state, 0, sizeof(state));
}

// 256-bit (32-byte) key
void ChaCha::Key(const void *key)
{
	state[0] = 0x6a09e667;
	state[1] = 0xbb67ae85;
	state[2] = 0x3c6ef372;
	state[3] = 0xa54ff53a;
	memcpy(&state[4], key, 32);
}

// 64-bit IV
void ChaCha::Begin(const u32 iv[2])
{
	state[12] = 0;
	state[13] = 0;
	state[14] = iv[0];
	state[15] = iv[1];
}

#define QUARTERROUND(a,b,c,d) \
  x[a] += x[b]; x[d] = ROL32(x[d] ^ x[a], 16); \
  x[c] += x[d]; x[b] = ROL32(x[b] ^ x[c], 12); \
  x[a] += x[b]; x[d] = ROL32(x[d] ^ x[a], 8); \
  x[c] += x[d]; x[b] = ROL32(x[b] ^ x[c], 7);

// Message with any number of bytes
void ChaCha::Crypt(const void *in, void *out, int bytes)
{
	const u8 *in8 = (const u8 *)in;
	u8 *out8 = (u8 *)out;

	while (bytes > 0)
	{
		u32 x[16];

		// Copy state into work registers
		for (int ii = 0; ii < 16; ++ii)
			x[ii] = state[ii];

		// Mix state for 12 rounds
		for (int round = 12; round > 0; round -= 2)
		{
			QUARTERROUND(0, 4, 8,  12)
			QUARTERROUND(1, 5, 9,  13)
			QUARTERROUND(2, 6, 10, 14)
			QUARTERROUND(3, 7, 11, 15)
			QUARTERROUND(0, 5, 10, 15)
			QUARTERROUND(1, 6, 11, 12)
			QUARTERROUND(2, 7, 8,  13)
			QUARTERROUND(3, 4, 9,  14)
		}

		// Add state to mixed state, little-endian
		for (int jj = 0; jj < 16; ++jj)
			x[jj] = getLE(x[jj] + state[jj]);

		// XOR words at a time
		const u32 *in32 = (const u32 *)in8;
		u32 *out32 = (u32 *)out8;
		int words = bytes / 4;
		if (words >= 16)
		{
			for (int kk = 0; kk < 16; ++kk)
				out32[kk] = in32[kk] ^ x[kk];
		}
		else
		{
			for (int kk = 0; kk < words; ++kk)
				out32[kk] = in32[kk] ^ x[kk];

			// XOR bytes at a time
			const u8 *x8b = (const u8 *)(x + words);
			const u8 *in8b = (const u8 *)(in8 + words*4);
			u8 *out8b = (u8 *)(out8 + words*4);

			switch (bytes % 4)
			{
			case 3: out8b[2] = in8b[2] ^ x8b[2];
			case 2: out8b[1] = in8b[1] ^ x8b[1];
			case 1: out8b[0] = in8b[0] ^ x8b[0];
			}
		}

		// Next block
		bytes -= 64;
		in8 += 64;
		out8 += 64;

		// Increment block counter
		state[12]++;
		if (!state[12]) state[13]++;
	}
}
