#include "CookieJar.hpp"
#include "Clock.hpp"
#include "Random.hpp"


CookieJar::CookieJar()
{
	// Generate the 512-bit key
	Random::ref()->generate(key, sizeof(key));
}

u32 CookieJar::Salsa6(u32 ip, u16 port, u32 epoch)
{
	u32 x[16];

	memcpy(x, key, sizeof(x));

	x[4] += ip;
	x[6] += epoch;
	x[8] += port;
	x[10] += epoch;

	// 6 rounds of Salsa20
	for (int ii = 6; ii > 0; ii -= 2)
	{
		x[4] ^= ROL32(x[0] + x[12], 7);
		x[8] ^= ROL32(x[4] + x[0], 9);
		x[12] ^= ROL32(x[8] + x[4], 13);
		x[0] ^= ROL32(x[12] + x[8], 18);
		x[9] ^= ROL32(x[5] + x[1], 7);
		x[13] ^= ROL32(x[9] + x[5], 9);
		x[1] ^= ROL32(x[13] + x[9], 13);
		x[5] ^= ROL32(x[1] + x[13], 18);
		x[14] ^= ROL32(x[10] + x[6], 7);
		x[2] ^= ROL32(x[14] + x[10], 9);
		x[6] ^= ROL32(x[2] + x[14], 13);
		x[10] ^= ROL32(x[6] + x[2], 18);
		x[3] ^= ROL32(x[15] + x[11], 7);
		x[7] ^= ROL32(x[3] + x[15], 9);
		x[11] ^= ROL32(x[7] + x[3], 13);
		x[15] ^= ROL32(x[11] + x[7], 18);
		x[1] ^= ROL32(x[0] + x[3], 7);
		x[2] ^= ROL32(x[1] + x[0], 9);
		x[3] ^= ROL32(x[2] + x[1], 13);
		x[0] ^= ROL32(x[3] + x[2], 18);
		x[6] ^= ROL32(x[5] + x[4], 7);
		x[7] ^= ROL32(x[6] + x[5], 9);
		x[4] ^= ROL32(x[7] + x[6], 13);
		x[5] ^= ROL32(x[4] + x[7], 18);
		x[11] ^= ROL32(x[10] + x[9], 7);
		x[8] ^= ROL32(x[11] + x[10], 9);
		x[9] ^= ROL32(x[8] + x[11], 13);
		x[10] ^= ROL32(x[9] + x[8], 18);
		x[12] ^= ROL32(x[15] + x[14], 7);
		x[13] ^= ROL32(x[12] + x[15], 9);
		x[14] ^= ROL32(x[13] + x[12], 13);
		x[15] ^= ROL32(x[14] + x[13], 18);
	}

	return x[0] ^ x[5] ^ x[10] ^ x[15];
}

u32 CookieJar::Generate(u32 ip, u16 port)
{
	u32 epoch = Clock::msec() / BIN_TIME;

	return (Salsa6(ip, port, epoch) << 4) | (epoch & BIN_MASK);
}

bool CookieJar::Verify(u32 ip, u16 port, u32 cookie)
{
	u32 epoch = Clock::msec() / BIN_TIME;

	u32 cookie_bin = cookie & BIN_MASK;

	// Attempt to reconstruct the epoch used to generate this cookie
	u32 cookie_epoch = (epoch & ~BIN_MASK) | cookie_bin;

	// If the cookie's bin is ahead of the bin in this interval, then
	// this cookie must be from the previous interval to be valid.
	if (cookie_bin > (epoch & BIN_MASK)) cookie_epoch -= BIN_COUNT;

	return (Salsa6(ip, port, cookie_epoch) << 4) == (cookie & ~BIN_MASK);
}
