#include "MMapFile.hpp"
#include "Logging.hpp"

#if defined(__linux__)
# include <sys/mman.h>
# include <sys/stat.h>
# include <fcntl.h>
#endif


MMapFile::MMapFile(const char *path)
{
	data = 0;
	offset = 0;
	len = 0;

#ifdef __linux__

	fd = open(path, O_RDONLY);
	if (fd == -1) { INANE("MMapFile") << "Unable to open file: " << path; return; }

	struct stat st;
	if (fstat(fd, &st) < 0) { INANE("MMapFile") << "Unable to stat file: " << path; return; }
	len = st.st_size;
	if (len == 0) return;

	data = (char *)mmap(0, len, PROT_READ, MAP_SHARED, fd, 0);
	if (data == MAP_FAILED) { INANE("MMapFile") << "Unable to mmap file: " << path; return; }

	close(fd);
	fd = -1;

#elif defined(_WIN32)

	hMapping = hFile = 0;

	hFile = CreateFileA(path, GENERIC_READ, FILE_SHARE_READ, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL|FILE_FLAG_RANDOM_ACCESS, 0);
	if (hFile == INVALID_HANDLE_VALUE) { INANE("MMapFile") << "Unable to open file: " << path; return; }

	len = GetFileSize(hFile, 0);
	if (len == -1) { INANE("MMapFile") << "Unable to stat file: " << path; return; }
	if (len == 0) return;

	hMapping = CreateFileMapping(hFile, 0, PAGE_READONLY, 0, 0, 0);
	if (!hMapping) { INANE("MMapFile") << "Unable to CreateFileMapping[" << GetLastError() << "]: " << path; return; }

	data = (char *)MapViewOfFile(hMapping, FILE_MAP_READ, 0, 0, 0);
	if (!data) { INANE("MMapFile") << "Unable to MapViewOfFile[" << GetLastError() << "]: " << path; return; }

#endif
}

MMapFile::~MMapFile()
{
#ifdef __linux__
	if (fd != -1) close(fd);
	if (data) munmap(data, len);
#elif defined(WIN32)
	if (data) UnmapViewOfFile(data);
	if (hMapping) CloseHandle(hMapping);
	if (hFile) CloseHandle(hFile);
#endif
}
