#include "common/ECC.hpp"
#include "common/Clock.hpp"
#include <iostream>
using namespace std;

int main()
{
	for (int ii = 0; ii < 10000; ++ii)
	{
		TwistedEdwardServer server;
		TwistedEdwardClient client;

		EdPrivateKey server_private_key;
		EdServerPublicKey server_public_key;
		EdClientPublicKey client_public_key;
		EdSharedSecret client_shared_secret;
		EdSharedSecret server_shared_secret;

		double t1, t2;

		cout << "OFFLINE PRECOMPUTATION: Generating server private and public keys" << endl;
		t1 = Clock::usec();
		server.GenerateOfflineStuff(&server_private_key, &server_public_key);
		t2 = Clock::usec();
		cout << "-- Operation completed in " << (t2 - t1) << " usec" << endl;

		cout << "ON SERVER START-UP: Loading server private key" << endl;
		t1 = Clock::usec();
		server.SetPrivateKey(&server_private_key);
		t2 = Clock::usec();
		cout << "-- Operation completed in " << (t2 - t1) << " usec" << endl;

		cout << "ON CLIENT START-UP: Loading server public key" << endl;
		t1 = Clock::usec();
		client.SetServerPublicKey(&server_public_key);
		t2 = Clock::usec();
		cout << "-- Operation completed in " << (t2 - t1) << " usec" << endl;

		cout << "CLIENT: ONLINE NOT TIME-CRITICAL: Computing shared secret" << endl;
		t1 = Clock::usec();
		client.ComputeSharedSecret(&client_public_key, &client_shared_secret);
		t2 = Clock::usec();
		cout << "-- Operation completed in " << (t2 - t1) << " usec" << endl;

		cout << "SERVER: ONLINE AND TIME-CRITICAL: Computing shared secret" << endl;
		t1 = Clock::usec();
		if (!server.ComputeSharedSecret(&client_public_key, &server_shared_secret))
		{
			cout << "FAILURE: Server rejected public key" << endl;
			break;
		}
		else
		{
			t2 = Clock::usec();
			cout << "SUCCESS: Server accepted public key" << endl;
		}
		cout << "-- Operation completed in " << (t2 - t1) << " usec" << endl;

		if (memcmp(server_shared_secret.Bytes, client_shared_secret.Bytes, sizeof(server_shared_secret.Bytes)))
		{
			cout << "FAILURE: Server shared secret did not match client shared secret" << endl;
			break;
		}
		else
		{
			cout << "SUCCESS: Server shared secret matched client shared secret" << endl;
		}
	}

	return 0;
}
